Index: main/stdtime/localtime.c
===================================================================
--- main/stdtime/localtime.c	(revision 65071)
+++ main/stdtime/localtime.c	(working copy)
@@ -46,6 +46,8 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <limits.h>
+#include <stdlib.h>
 #include <fcntl.h>
 #ifdef DEBUG
 #include <stdio.h>
@@ -1017,9 +1019,17 @@
 static const struct state *ast_tzset(const char *zone)
 {
 	struct state *sp;
+	char buf[FILENAME_MAX + 1] = {0};
 
-	if (ast_strlen_zero(zone))
-		zone = "/etc/localtime";
+	if (ast_strlen_zero(zone)) {
+		/* Store the full pathname in order to notice when the timezone is
+		 * changed simply by changing the target of a symlink.
+		 */
+		if (realpath("/etc/localtime", buf) != NULL)
+			zone = buf;
+		else
+			zone = "/etc/localtime";
+	}
 
 	AST_LIST_LOCK(&zonelist);
 	AST_LIST_TRAVERSE(&zonelist, sp, list) {
@@ -1030,6 +1040,10 @@
 	}
 	AST_LIST_UNLOCK(&zonelist);
 
+	/* Resolve the path if we haven't already done so */
+	if (buf[0] == 0 && realpath(zone, buf) != NULL)
+		zone = buf;
+
 	if (!(sp = ast_calloc(1, sizeof *sp)))
 		return NULL;
 
